package net.wanji.web.service.scheme.impl;

import cn.hutool.core.collection.CollectionUtil;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.databus.dao.entity.BaseCrossSchedulesPO;
import net.wanji.databus.dao.entity.CrossPlanPO;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.mapper.BaseCrossPlanMapper;
import net.wanji.databus.dao.mapper.BaseCrossSchedulesMapper;
import net.wanji.databus.dao.mapper.BaseCrossSchemeMapper;
import net.wanji.databus.dao.mapper.BaseCrossSectionMapper;
import net.wanji.databus.dto.RunningPlanDTO;
import net.wanji.databus.dao.mapper.CrossSchedulesPlanMapper;
import net.wanji.databus.dao.entity.CrossSchedulesPlanPO;
import net.wanji.web.service.scheme.RunningPlanService;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/1/28 16:41
 */
@Service
public class RunningPlanServiceImpl implements RunningPlanService {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    private final BaseCrossPlanMapper baseCrossPlanMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final CrossSchedulesPlanMapper crossSchedulesPlanMapper;

    public RunningPlanServiceImpl(BaseCrossPlanMapper baseCrossPlanMapper, BaseCrossSchemeMapper baseCrossSchemeMapper,
                                  BaseCrossSectionMapper baseCrossSectionMapper, BaseCrossSchedulesMapper baseCrossSchedulesMapper,
                                  CrossSchedulesPlanMapper crossSchedulesPlanMapper) {
        this.baseCrossPlanMapper = baseCrossPlanMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.crossSchedulesPlanMapper = crossSchedulesPlanMapper;
    }

    @Override
    @Transactional
    public void saveRunningPlan(RunningPlanDTO runningPlanDTO) throws ParseException {
        String crossId = runningPlanDTO.getCrossId();
        // 清空之前计划
        baseCrossPlanMapper.deleteByCrossId(crossId);
        baseCrossSectionMapper.deleteByCrossId(crossId);
        baseCrossSchedulesMapper.deleteByCrossId(crossId);
        crossSchedulesPlanMapper.deleteByCrossId(crossId);
        // 保存计划数据
        saveCrossPlan(runningPlanDTO, crossId);
        // 保存调度数据
        saveCrossSchedules(runningPlanDTO, crossId);
    }

    @Override
    public RunningPlanDTO listRunningPlan(CrossIdDTO crossIdDTO) {
        RunningPlanDTO runningPlanDTO = new RunningPlanDTO();
        String crossId = crossIdDTO.getCrossId();
        runningPlanDTO.setCrossId(crossId);
        // 构造dailyPlanList
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = buildDailyPlanList(crossId);
        runningPlanDTO.setDailyPlanList(dailyPlanList);
        // 构造schedulesPlanList
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = buildSchedulesPlanList(crossId);
        runningPlanDTO.setSchedulesPlanList(schedulesPlanList);

        return runningPlanDTO;
    }

    private List<RunningPlanDTO.SchedulesPlanListElement> buildSchedulesPlanList(String crossId) {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = new ArrayList<>();
        List<BaseCrossSchedulesPO> baseCrossSchedulesPOList = baseCrossSchedulesMapper.selectByCrossId(crossId);
        for (BaseCrossSchedulesPO baseCrossSchedulesPO : baseCrossSchedulesPOList) {
            String scheduleNo = baseCrossSchedulesPO.getScheduleNo();
            String name = baseCrossSchedulesPO.getName();
            Integer crossSchedulesPOId = baseCrossSchedulesPO.getId();
            RunningPlanDTO.SchedulesPlanListElement schedulesPlan = new RunningPlanDTO.SchedulesPlanListElement();
            schedulesPlan.setScheduleNo(scheduleNo);
            schedulesPlan.setName(name);
            schedulesPlan.setId(crossSchedulesPOId);
            schedulesPlan.setStatus(baseCrossSchedulesPO.getStatus());
            // 构造execList
            List<RunningPlanDTO.ExecListElement> execList = buildExecList(crossId, scheduleNo);
            schedulesPlan.setExecList(execList);
            schedulesPlanList.add(schedulesPlan);
        }
        return schedulesPlanList;
    }

    private List<RunningPlanDTO.ExecListElement> buildExecList(String crossId, String scheduleNo) {
        List<RunningPlanDTO.ExecListElement> execList = new ArrayList<>();
        // 获取调度ID
        BaseCrossSchedulesPO schedulesPO = baseCrossSchedulesMapper.selectByCrossIdAndScheduleNo(crossId, scheduleNo);
        Integer schedulesId = schedulesPO.getId();
        // 根据路口ID、调度ID和计划ID查询调度计划关系
        List<CrossSchedulesPlanPO> crossSchedulesPlanPOList =
                crossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, schedulesId);
        Map<Integer, List<CrossSchedulesPlanPO>> collectMap = crossSchedulesPlanPOList.stream()
                .collect(Collectors.groupingBy(CrossSchedulesPlanPO::getPlanId));
        for (Map.Entry<Integer, List<CrossSchedulesPlanPO>> entry : collectMap.entrySet()) {
            RunningPlanDTO.ExecListElement execListElement = new RunningPlanDTO.ExecListElement();
            List<Integer> week = new ArrayList<>();
            List<String> specialDateList = new ArrayList<>();
            Integer planId = entry.getKey();
            List<CrossSchedulesPlanPO> secondList = entry.getValue();
            for (CrossSchedulesPlanPO crossSchedulesPlanPO : secondList) {
                Integer day = crossSchedulesPlanPO.getWeek();
                if (day == 0) { // 特殊日期
                    Date specialDate = crossSchedulesPlanPO.getSpecialDate();
                    specialDateList.add(sdf.format(specialDate));
                } else {
                    week.add(day);
                }
            }
            execListElement.setWeek(week);
            execListElement.setSpecialDateList(specialDateList);
            CrossPlanPO crossPlanPO = baseCrossPlanMapper.selectById(planId);
            execListElement.setPlanName(crossPlanPO.getName());
            execListElement.setPlanId(crossPlanPO.getId());
            execList.add(execListElement);
        }
        return execList;
    }

    private List<RunningPlanDTO.DailyPlanListElement> buildDailyPlanList(String crossId) {
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = new ArrayList<>();
        dailyPlanList = baseCrossPlanMapper.selectDailyPlanList(crossId);
        if (CollectionUtil.isEmpty(dailyPlanList)) {
            List<CrossPlanPO> crossPlanPOList = baseCrossPlanMapper.selectByCrossId(crossId);
            if (CollectionUtil.isNotEmpty(crossPlanPOList)) {
                for (CrossPlanPO crossPlanPO : crossPlanPOList) {
                    RunningPlanDTO.DailyPlanListElement dailyPlan = new RunningPlanDTO.DailyPlanListElement();
                    dailyPlan.setPlanNo(crossPlanPO.getPlanNo());
                    dailyPlan.setName(crossPlanPO.getName());
                    dailyPlan.setTimeList(new ArrayList<>());
                    dailyPlanList.add(dailyPlan);
                }
            }
        } else {
            for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
                List<RunningPlanDTO.TimeListElement> timeList = dailyPlan.getTimeList();
                for (RunningPlanDTO.TimeListElement timeListElement : timeList) {
                    String[] startEndTime = new String[2];
                    String startTime = timeListElement.getStartTime();
                    String endTime = timeListElement.getEndTime();
                    startEndTime[0] = startTime;
                    startEndTime[1] = endTime;
                    timeListElement.setStartEndTime(startEndTime);
                }
            }
        }
        return dailyPlanList;
    }

    private void saveCrossSchedules(RunningPlanDTO runningPlanDTO, String crossId) throws ParseException {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            // 更新调度表获取调度ID
            BaseCrossSchedulesPO baseCrossSchedulesPO = new BaseCrossSchedulesPO();
            baseCrossSchedulesPO.setScheduleNo(schedulesPlan.getScheduleNo());
            baseCrossSchedulesPO.setName(schedulesPlan.getName());
            baseCrossSchedulesPO.setStatus(0); // 新增调度状态为未执行
            baseCrossSchedulesPO.setCrossId(crossId);
            baseCrossSchedulesMapper.insertOne(baseCrossSchedulesPO);
            Integer crossSchedulesId = baseCrossSchedulesPO.getId();
            // 遍历执行的日计划列表
            List<RunningPlanDTO.ExecListElement> execList = schedulesPlan.getExecList();
            for (RunningPlanDTO.ExecListElement execListElement : execList) {
                // 根据路口ID和计划名获取计划ID
                String planName = execListElement.getPlanName();
                CrossPlanPO crossPlanPO = baseCrossPlanMapper.selectByCrossIdAndName(crossId, planName);
                if (crossPlanPO != null) {
                    Integer planId = crossPlanPO.getId();
                    // 更新调度计划关系表
                    updateCrossSchedulesPlan(crossId, execListElement, crossSchedulesId, planId);
                }
            }
        }
    }

    private void updateCrossSchedulesPlan(String crossId, RunningPlanDTO.ExecListElement schedulesPlan,
                                          Integer crossSchedulesId, Integer planId) throws ParseException {
        List<Integer> week = schedulesPlan.getWeek();
        List<String> specialDateList = schedulesPlan.getSpecialDateList();
        if (CollectionUtil.isNotEmpty(specialDateList)) {
            for (String date : specialDateList) {
                CrossSchedulesPlanPO crossSchedulesPlanPO = initData(crossId, crossSchedulesId, planId);
                crossSchedulesPlanPO.setSpecialDate(sdf.parse(date));
                crossSchedulesPlanPO.setWeek(0);
                crossSchedulesPlanMapper.insertOne(crossSchedulesPlanPO);
            }
        }
        if (CollectionUtil.isNotEmpty(week)) {
            for (Integer day : week) {
                CrossSchedulesPlanPO crossSchedulesPlanPO = initData(crossId, crossSchedulesId, planId);
                crossSchedulesPlanPO.setSpecialDate(new Date(0));
                crossSchedulesPlanPO.setWeek(day);
                crossSchedulesPlanMapper.insertOne(crossSchedulesPlanPO);
            }
        }
    }

    @NotNull
    private static CrossSchedulesPlanPO initData(String crossId, Integer crossSchedulesId, Integer planId) {
        CrossSchedulesPlanPO crossSchedulesPlanPO = new CrossSchedulesPlanPO();
        crossSchedulesPlanPO.setCrossId(crossId);
        crossSchedulesPlanPO.setSchedulesId(crossSchedulesId);
        crossSchedulesPlanPO.setPlanId(planId);
        return crossSchedulesPlanPO;
    }

    private void saveCrossPlan(RunningPlanDTO runningPlanDTO, String crossId) {
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = runningPlanDTO.getDailyPlanList();
        for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
            // 更新计划表获取计划ID
            CrossPlanPO crossPlanPO = new CrossPlanPO();
            String planNo = dailyPlan.getPlanNo();
            try {
                Integer.parseInt(planNo);
            } catch (Exception e) {
                throw new RuntimeException("计划编号必须为数字");
            }
            crossPlanPO.setPlanNo(planNo);
            crossPlanPO.setName(dailyPlan.getName());
            crossPlanPO.setCrossId(crossId);
            baseCrossPlanMapper.insertOne(crossPlanPO);
            Integer crossPlanId = crossPlanPO.getId();
            List<RunningPlanDTO.TimeListElement> timeList = dailyPlan.getTimeList();
            for (RunningPlanDTO.TimeListElement time : timeList) {
                // 根据路口ID和方案名获取方案ID
                String schemeName = time.getSchemeName();
                BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectByCrossIdAndSchemeName(crossId, schemeName);
                Integer schemeId = baseCrossSchemePO.getId();
                // 更新时段表
                updateCrossSection(crossId, crossPlanId, schemeId, time);
            }
        }
    }

    private void updateCrossSection(String crossId, Integer crossPlanId, Integer schemeId,
                                    RunningPlanDTO.TimeListElement time) {
        CrossSectionPO crossSectionPO = new CrossSectionPO();
        String[] startEndTime = time.getStartEndTime();
        String startTime = startEndTime[0];
        String endTime = startEndTime[1];
        String s = startTime + endTime;
        if ("23:59".equals(endTime)) {
            endTime = "24:00";
        }
        String sectionNo = s.replaceAll(":", "");
        crossSectionPO.setSectionNo(sectionNo);
        crossSectionPO.setStartTime(startTime);
        crossSectionPO.setEndTime(endTime);
        crossSectionPO.setCrossId(crossId);
        crossSectionPO.setPlanId(crossPlanId);
        crossSectionPO.setControlMode(1); // todo 暂时只有定周期
        crossSectionPO.setSchemeId(schemeId);
        baseCrossSectionMapper.insertOne(crossSectionPO);
    }
}
